@@ -482,9 +482,12 @@ var GraphDataWindow = {
                 id: 'combineMenu',
                 menu: [
                   {text: 'Sum', handler: this.applyFuncToAll('sumSeries')},
-                  {text: 'Difference', handler: this.applyFuncToAll('diffSeries')},
                   {text: 'Average', handler: this.applyFuncToAll('averageSeries')},
-                  {text: 'Sum using wildcards', handler: this.applyFuncToEachWithInput('sumSeriesWithWildcards', 'Please enter a comma separated list of numbers specifying the locations in the name to place wildcards')}
+                  {text: 'Sum using wildcards', handler: this.applyFuncToEachWithInput('sumSeriesWithWildcards', 'Please enter a comma separated list of numbers specifying the locations in the name to place wildcards')},
+                  {text: 'Average using wildcards', handler: this.applyFuncToEachWithInput('averageSeriesWithWildcards', 'Please enter a comma separated list of numbers specifying the locations in the name to place wildcards')},
+                  {text: 'Min Values', handler: this.applyFuncToAll('minSeries')},
+                  {text: 'Max Values', handler: this.applyFuncToAll('maxSeries')},
+                  {text: 'Group', handler: this.applyFuncToAll('group')}
                 ]
               }, {
                 text: 'Transform',
@@ -493,16 +496,20 @@ var GraphDataWindow = {
                   {text: 'Offset', handler: this.applyFuncToEachWithInput('offset', 'Please enter the value to offset Y-values by')},
                   {text: 'Derivative', handler: this.applyFuncToEach('derivative')},
                   {text: 'Integral', handler: this.applyFuncToEach('integral')},
-                  {text: 'Non-negative Derivative', handler: this.applyFuncToEachWithInput('nonNegativeDerivative', "Please enter a maximum value if this metric is a wrapping counter (or just leave this blank)",true)},
+                  {text: 'Non-negative Derivative', handler: this.applyFuncToEachWithInput('nonNegativeDerivative', "Please enter a maximum value if this metric is a wrapping counter (or just leave this blank)", {allowBlank: true})},
                   {text: 'Log', handler: this.applyFuncToEachWithInput('log', 'Please enter a base')},
-                  {text: 'timeShift', handler: this.applyFuncToEachWithInput('timeShift', 'Shift this metric ___ back in time')}
+                  {text: 'timeShift', handler: this.applyFuncToEachWithInput('timeShift', 'Shift this metric ___ back in time (examples: 10min, 7d, 2w)', {quote: true})},
+                  {text: 'Summarize', handler: this.applyFuncToEachWithInput('summarize', 'Please enter a summary interval (examples: 10min, 1h, 7d)', {quote: true})},
+                  {text: 'Hit Count', handler: this.applyFuncToEachWithInput('hitcount', 'Please enter a summary interval (examples: 10min, 1h, 7d)', {quote: true})}
                 ]
               }, {
                 text: 'Calculate',
                 menu: [
                   {text: 'Moving Average', handler: this.applyFuncToEachWithInput('movingAverage', 'Moving average for the last ___ data points')},
                   {text: 'Moving Standard Deviation', handler: this.applyFuncToEachWithInput('stdev', 'Moving standard deviation for the last ___ data points')},
-                  {text: 'As Percent', handler: this.applyFuncToEachWithInput('asPercent', 'Please enter the value that corresponds to 100%')}
+                  {text: 'As Percent', handler: this.applyFuncToEachWithInput('asPercent', 'Please enter the value that corresponds to 100%')},
+                  {text: 'Difference (of 2 series)', handler: this.applyFuncToAll('diffSeries')},
+                  {text: 'Ratio (of 2 series)', handler: this.applyFuncToAll('divideSeries')}
                 ]
               }, {
                 text: 'Filter',
@@ -510,6 +517,8 @@ var GraphDataWindow = {
                   {text: 'Most Deviant', handler: this.applyFuncToEachWithInput('mostDeviant', 'Draw the ___ metrics with the highest standard deviation')},
                   {text: 'Highest Current Value', handler: this.applyFuncToEachWithInput('highestCurrent', 'Draw the ___ metrics with the highest current value')},
                   {text: 'Lowest Current Value', handler: this.applyFuncToEachWithInput('lowestCurrent', 'Draw the ___ metrics with the lowest current value')},
+                  {text: 'Highest Maximum Value', handler: this.applyFuncToEachWithInput('highestMax', 'Draw the ___ metrics with the highest maximum value')},
+                  {text: 'Nth Percentile Value', handler: this.applyFuncToEachWithInput('nPercentile', 'Draw the ___th Percentile for each metric.')},
                   {text: 'Current Value Above', handler: this.applyFuncToEachWithInput('currentAbove', 'Draw all metrics whose current value is above ___')},
                   {text: 'Current Value Below', handler: this.applyFuncToEachWithInput('currentBelow', 'Draw all metrics whose current value is below ___')},
                   {text: 'Highest Average Value', handler: this.applyFuncToEachWithInput('highestAverage', 'Draw the ___ metrics with the highest average value')},
@@ -520,18 +529,20 @@ var GraphDataWindow = {
                   {text: 'Maximum Value Below', handler: this.applyFuncToEachWithInput('maximumBelow', 'Draw all metrics whose maximum value is below ___')},
                   {text: 'sortByMaxima', handler: this.applyFuncToEach('sortByMaxima')},
                   {text: 'sortByMinima', handler: this.applyFuncToEach('sortByMinima')},
-                  {text: 'limit', handler: this.applyFuncToEachWithInput('limit', 'Limit to first ___ of a list of metrics')}
+                  {text: 'limit', handler: this.applyFuncToEachWithInput('limit', 'Limit to first ___ of a list of metrics')},
+                  {text: 'Exclude', handler: this.applyFuncToEachWithInput('exclude', 'Exclude metrics that match a regular expression')}
                 ]
               }, {
                 text: 'Special',
                 menu: [
-                  {text: 'Set Legend Name', handler: this.applyFuncToEachWithInput('alias', 'Enter a legend label for this graph target')},
+                  {text: 'Set Legend Name', handler: this.applyFuncToEachWithInput('alias', 'Enter a legend label for this graph target', {quote: true})},
                   {text: 'Aggregate By Sum', handler: this.applyFuncToEach('cumulative')},
                   {text: 'Draw non-zero As Infinite', handler: this.applyFuncToEach('drawAsInfinite')},
                   {text: 'Line Width', handler: this.applyFuncToEachWithInput('lineWidth', 'Please enter a line width for this graph target')},
                   {text: 'Dashed Line', handler: this.applyFuncToEach('dashed')},
                   {text: 'Keep Last Value', handler: this.applyFuncToEach('keepLastValue')},
-                  {text: 'Substring', handler: this.applyFuncToEachWithInput('substr', 'Enter a starting position')}
+                  {text: 'Substring', handler: this.applyFuncToEachWithInput('substr', 'Enter a starting position')},
+                  {text: 'Add Threshold Line', handler: this.applyFuncToEachWithInput('threshold', 'Enter a threshold value')}
                 ]
               }
             ]
@@ -643,14 +654,18 @@ var GraphDataWindow = {
     return applyFunc;
   },
 
-  applyFuncToEachWithInput: function (funcName, question, allowBlank) {
+  applyFuncToEachWithInput: function (funcName, question, options) {
+    if (options == null) {
+      options = {};
+    }
+
     function applyFunc() {
       Ext.MessageBox.prompt(
         "Input Required", //title
         question, //message
         function (button, inputValue) { //handler
-          if (button == 'ok' && (allowBlank || inputValue != '')) {
-            if (funcName == 'alias' && inputValue != '') { //SPECIAL CASE HACK
+          if (button == 'ok' && (options.allowBlank || inputValue != '')) {
+            if (options.quote) {
               inputValue = '"' + inputValue + '"';
             }
             this.applyFuncToEach(funcName, inputValue)();
@@ -979,10 +994,27 @@ function createOptionsMenu() {
     ]
   });
 
+  var yAxisUnitMenu = new Ext.menu.Menu({
+    items: [
+      menuRadioItem("yUnit", "Standard", "yUnitSystem", "si"),
+      menuRadioItem("yUnit", "Binary", "yUnitSystem", "binary"),
+      menuRadioItem("yUnit", "None", "yUnitSystem", "none")
+    ]
+  });
+
+  var yAxisMenu = new Ext.menu.Menu({
+    items: [
+      menuInputItem("Label", "vtitle"),
+      menuInputItem("Minimum", "yMin"),
+      menuInputItem("Maximum", "yMax"),
+      {text: "Unit", menu: yAxisUnitMenu}
+    ]
+  });
+
   return new Ext.menu.Menu({
     items: [
       menuInputItem("Graph Title", "title"),
-      menuInputItem("Y Axis Label", "vtitle"),
+      {text: "Y Axis", menu: yAxisMenu},
       menuInputItem("Line Thickness", "lineWidth"),
       {text: "Area Mode", menu: areaMenu},
       menuCheckItem("Alpha Masking", "template", "alphas"),
