@@ -18,6 +18,7 @@ from django.conf import settings
 from graphite.account.models import Profile
 from graphite.util import getProfile, getProfileByUsername, defaultUser, json
 from graphite.logger import log
+from graphite.storage import STORE, LOCAL_STORE
 
 try:
   import cPickle as pickle
@@ -34,7 +35,7 @@ def context_view(request):
 
     for metric in request.GET.getlist('metric'):
       try:
-        context = settings.STORE.get(metric).context
+        context = STORE.get(metric).context
       except:
         contexts.append({ 'metric' : metric, 'error' : 'failed to retrieve context', 'traceback' : traceback.format_exc() })
       else:
@@ -51,7 +52,7 @@ def context_view(request):
     newContext = dict( item for item in request.POST.items() if item[0] != 'metric' )
 
     for metric in request.POST.getlist('metric'):
-      settings.STORE.get(metric).updateContext(newContext)
+      STORE.get(metric).updateContext(newContext)
 
     return HttpResponse('{ "success" : true }', mimetype='text/json')
 
@@ -78,9 +79,9 @@ def find_view(request):
     base_path = ''
 
   if local_only:
-    store = settings.LOCAL_STORE
+    store = LOCAL_STORE
   else:
-    store = settings.STORE
+    store = STORE
 
   matches = list( store.find(query) )
 
@@ -96,10 +97,15 @@ def find_view(request):
     response = HttpResponse(content, mimetype='application/pickle')
 
   elif format == 'completer':
-    if len(matches) == 1 and (not matches[0].isLeaf()) and query == matches[0].metric_path + '*': # auto-complete children
-      matches = list( store.find(query + '.*') )
+    #if len(matches) == 1 and (not matches[0].isLeaf()) and query == matches[0].metric_path + '*': # auto-complete children
+    #  matches = list( store.find(query + '.*') )
+    results = [ dict(path=node.metric_path, name=node.name) for node in matches ]
 
-    content = json.dumps({ 'metrics' : [ dict(path=node.metric_path, name=node.name) for node in matches ] })
+    if len(results) > 1 and wildcards:
+      wildcardNode = {'name' : '*'}
+      results.append(wildcardNode)
+
+    content = json.dumps({ 'metrics' : results })
     response = HttpResponse(content, mimetype='text/json')
 
   else:
@@ -110,6 +116,30 @@ def find_view(request):
   return response
 
 
+def expand_view(request):
+  "View for expanding a pattern into matching metric paths"
+  local_only = int( request.REQUEST.get('local', 0) )
+
+  if local_only:
+    store = LOCAL_STORE
+  else:
+    store = STORE
+
+  results = set()
+  for query in request.REQUEST.getlist('query'):
+    for node in store.find(query):
+      results.add( node.metric_path )
+
+  result = {
+    'results' : sorted(results)
+  }
+
+  response = HttpResponse(json.dumps(result), mimetype='text/json')
+  response['Pragma'] = 'no-cache'
+  response['Cache-Control'] = 'no-cache'
+  return response
+
+
 def tree_json(nodes, base_path, wildcards=False, contexts=False):
   results = []
 
@@ -165,10 +195,10 @@ def tree_json(nodes, base_path, wildcards=False, contexts=False):
 
 def pickle_nodes(nodes, contexts=False):
   if contexts:
-    return pickle.dumps([ { 'metric_path' : n.metric_path, 'isLeaf' : n.isLeaf(), 'context' : n.context } for n in nodes ])
+    return pickle.dumps([ { 'metric_path' : n.metric_path, 'isLeaf' : n.isLeaf(), 'intervals' : n.intervals, 'context' : n.context } for n in nodes ])
 
   else:
-    return pickle.dumps([ { 'metric_path' : n.metric_path, 'isLeaf' : n.isLeaf() } for n in nodes ])
+    return pickle.dumps([ { 'metric_path' : n.metric_path, 'isLeaf' : n.isLeaf(), 'intervals' : n.intervals} for n in nodes ])
 
 
 def any(iterable): #python2.4 compatibility
