@@ -12,9 +12,11 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License."""
 
-from graphite.render.datatypes import TimeSeries
+from graphite.render.datalib import TimeSeries, timestamp
+from graphite.render.attime import parseTimeOffset
 from itertools import izip
 import math
+import re
 
 #Utility functions
 def safeSum(values):
@@ -45,6 +47,16 @@ def safeLast(values):
   for v in reversed(values):
     if v is not None: return v
 
+def safeMin(values):
+  safeValues = [v for v in values if v is not None]
+  if safeValues:
+    return min(safeValues)
+
+def safeMax(values):
+  safeValues = [v for v in values if v is not None]
+  if safeValues:
+    return max(safeValues)
+
 def lcm(a,b):
   'least common multiple'
   if a == b: return a
@@ -71,7 +83,7 @@ def normalize(seriesLists):
 #in practice this *shouldn't* matter because all series will cover
 #the same interval, despite having possibly different steps...
 
-def sumSeries(*seriesLists):
+def sumSeries(requestContext, *seriesLists):
   try:
     (seriesList,start,end,step) = normalize(seriesLists)
   except:
@@ -83,7 +95,7 @@ def sumSeries(*seriesLists):
   series.pathExpression = name
   return [series]
 
-def sumSeriesWithWildcards(seriesList, *position):
+def sumSeriesWithWildcards(requestContext, seriesList, *position): #XXX
   if type(position) is int:
     positions = [position]
   else:
@@ -92,13 +104,13 @@ def sumSeriesWithWildcards(seriesList, *position):
   for series in seriesList:
     newname = '.'.join(map(lambda x: x[1], filter(lambda i: i[0] not in positions, enumerate(series.name.split('.')))))
     if newname in newSeries.keys():
-      newSeries[newname] = sumSeries((series, newSeries[newname]))[0]
+      newSeries[newname] = sumSeries(requestContext, (series, newSeries[newname]))[0]
     else:
       newSeries[newname] = series
     newSeries[newname].name = newname
   return newSeries.values()
 
-def averageSeriesWithWildcards(seriesList, *position):
+def averageSeriesWithWildcards(requestContext, seriesList, *position): #XXX
   if type(position) is int:
     positions = [position]
   else:
@@ -111,11 +123,11 @@ def averageSeriesWithWildcards(seriesList, *position):
       matchedList[newname] = []
     matchedList[newname].append(series)
   for name in matchedList.keys():
-    result.append(averageSeries((matchedList[name]))[0])
+    result.append( averageSeries(requestContext, (matchedList[name]))[0] )
     result[-1].name = name
   return result
 
-def diffSeries(*seriesLists):
+def diffSeries(requestContext, *seriesLists):
   (seriesList,start,end,step) = normalize(seriesLists)
   name = "diffSeries(%s)" % ','.join(set([s.pathExpression for s in seriesList]))
   values = ( safeDiff(row) for row in izip(*seriesList) )
@@ -123,7 +135,7 @@ def diffSeries(*seriesLists):
   series.pathExpression = name
   return [series]
 
-def averageSeries(*seriesLists):
+def averageSeries(requestContext, *seriesLists):
   (seriesList,start,end,step) = normalize(seriesLists)
   #name = "averageSeries(%s)" % ','.join((s.name for s in seriesList))
   name = "averageSeries(%s)" % ','.join(set([s.pathExpression for s in seriesList]))
@@ -132,7 +144,25 @@ def averageSeries(*seriesLists):
   series.pathExpression = name
   return [series]
 
-def keepLastValue(seriesList):
+def minSeries(requestContext, *seriesLists):
+  (seriesList, start, end, step) = normalize(seriesLists)
+  pathExprs = list( set([s.pathExpression for s in seriesList]) )
+  name = "minSeries(%s)" % ','.join(pathExprs)
+  values = ( safeMin(row) for row in izip(*seriesList) )
+  series = TimeSeries(name, start, end, step, values)
+  series.pathExpression = name
+  return [series]
+
+def maxSeries(requestContext, *seriesLists):
+  (seriesList, start, end, step) = normalize(seriesLists)
+  pathExprs = list( set([s.pathExpression for s in seriesList]) )
+  name = "maxSeries(%s)" % ','.join(pathExprs)
+  values = ( safeMax(row) for row in izip(*seriesList) )
+  series = TimeSeries(name, start, end, step, values)
+  series.pathExpression = name
+  return [series]
+
+def keepLastValue(requestContext, seriesList):
   for series in seriesList:
     series.name = "keepLastValue(%s)" % (series.name)
     for i,value in enumerate(series):
@@ -141,7 +171,7 @@ def keepLastValue(seriesList):
       series[i] = value
   return seriesList
 
-def asPercent(seriesList1,seriesList2orNumber):
+def asPercent(requestContext, seriesList1, seriesList2orNumber):
   assert len(seriesList1) == 1, "asPercent series arguments must reference *exactly* 1 series"
   series1 = seriesList1[0]
   if type(seriesList2orNumber) is list:
@@ -167,14 +197,43 @@ def asPercent(seriesList1,seriesList2orNumber):
   series.pathExpression = name
   return [series]
 
-def scale(seriesList,factor):
+
+def divideSeries(requestContext, dividendSeriesList, divisorSeriesList):
+  if len(divisorSeriesList) != 1:
+    raise ValueError("divideSeries second argument must reference exactly 1 series")
+
+  divisorSeries = divisorSeriesList[0]
+  results = []
+
+  for dividendSeries in dividendSeriesList:
+    name = "divideSeries(%s,%s)" % (dividendSeries.name, divisorSeries.name)
+    bothSeries = (dividendSeries, divisorSeries)
+    step = reduce(lcm,[s.step for s in bothSeries])
+
+    for s in bothSeries:
+      s.consolidate( step / s.step )
+
+    start = min([s.start for s in bothSeries])
+    end = max([s.end for s in bothSeries])
+    end -= (end - start) % step
+
+    values = ( safeDiv(v1,v2) for v1,v2 in izip(*bothSeries) )
+
+    quotientSeries = TimeSeries(name, start, end, step, values)
+    quotientSeries.pathExpression = name
+    results.append(quotientSeries)
+
+  return results
+
+
+def scale(requestContext, seriesList, factor):
   for series in seriesList:
     series.name = "scale(%s,%.1f)" % (series.name,float(factor))
     for i,value in enumerate(series):
       series[i] = safeMul(value,factor)
   return seriesList
 
-def offset(seriesList,factor):
+def offset(requestContext, seriesList, factor):
   for series in seriesList:
     series.name = "offset(%s,%.1f)" % (series.name,float(factor))
     for i,value in enumerate(series):
@@ -182,35 +241,37 @@ def offset(seriesList,factor):
         series[i] = value + factor
   return seriesList
 
-def movingAverage(seriesList,time):
-  count = 0
-  for series in seriesList:
-    movAvg = TimeSeries("movingAverage(%s,%.1f)" % (series.name,float(time)),series.start,series.end,series.step,[])
-    movAvg.pathExpression = "movingAverage(%s,%.1f)" % (series.name,float(time))
-    avg = safeDiv(safeSum(series[:time]), time)
-    movAvg.append(avg)
-    for (index, el) in enumerate(series[time:]):
-      if el is None:
-        continue
-      toDrop = series[index]
-      if toDrop is None:
-        toDrop = 0
-      s = safeSum([safeMul(time, avg), el, -toDrop])
-      avg = safeDiv(s, time)
-      movAvg.append(avg)
-    for i in range(0, time-1):
-      movAvg.insert(0, None)
-    seriesList[count] = movAvg
-    count = count + 1
+def movingAverage(requestContext, seriesList, windowSize):
+  for seriesIndex, series in enumerate(seriesList):
+    newName = "movingAverage(%s,%.1f)" % (series.name, float(windowSize))
+    newSeries = TimeSeries(newName, series.start, series.end, series.step, [])
+    newSeries.pathExpression = newName
+
+    windowIndex = windowSize - 1
+
+    for i in range( len(series) ):
+      if i < windowIndex: # Pad the beginning with None's since we don't have enough data
+        newSeries.append( None )
+
+      else:
+        window = series[i - windowIndex : i + 1]
+        nonNull = [ v for v in window if v is not None ]
+        if nonNull:
+          newSeries.append( sum(nonNull) / len(nonNull) )
+        else:
+          newSeries.append(None)
+
+    seriesList[ seriesIndex ] = newSeries
+
   return seriesList
 
-def cumulative(seriesList):
+def cumulative(requestContext, seriesList):
   for series in seriesList:
     series.consolidationFunc = 'sum'
     series.name = 'cumulative(%s)' % series.name
   return seriesList
 
-def derivative(seriesList):
+def derivative(requestContext, seriesList):
   results = []
   for series in seriesList:
     newValues = []
@@ -228,7 +289,7 @@ def derivative(seriesList):
     results.append(newSeries)
   return results
 
-def integral(seriesList):
+def integral(requestContext, seriesList):
   results = []
   for series in seriesList:
     newValues = []
@@ -245,38 +306,45 @@ def integral(seriesList):
     results.append(newSeries)
   return results
 
-def nonNegativeDerivative(seriesList, maxValue=None): # useful for watching counter metrics that occasionally wrap
+
+def nonNegativeDerivative(requestContext, seriesList, maxValue=None):
   results = []
+
   for series in seriesList:
     newValues = []
     prev = None
+
     for val in series:
-      if None in (prev,val):
+      if None in (prev, val):
         newValues.append(None)
         prev = val
         continue
+
       diff = val - prev
       if diff >= 0:
         newValues.append(diff)
       elif maxValue is not None and maxValue >= val:
-        newValues.append( prev + (maxValue - val) )
+        newValues.append( (maxValue - prev) + val  + 1 )
       else:
         newValues.append(None)
+
       prev = val
+
     newName = "nonNegativeDerivative(%s)" % series.name
     newSeries = TimeSeries(newName, series.start, series.end, series.step, newValues)
     newSeries.pathExpression = newName
     results.append(newSeries)
+
   return results
 
 
-def alias(seriesList,newName):
+def alias(requestContext, seriesList, newName):
   for series in seriesList:
     series.name = newName
   return seriesList
 
 
-def substr(seriesList, start=0, stop=0):
+def substr(requestContext, seriesList, start=0, stop=0):
   for series in seriesList:
     left = series.name.rfind('(') + 1
     right = series.name.find(')')
@@ -289,13 +357,16 @@ def substr(seriesList, start=0, stop=0):
       series.name = '.'.join(cleanName.split('.')[int(start):int(stop):])
   return seriesList
 
-def log(seriesList, base=10):
+
+def log(requestContext, seriesList, base=10):
   results = []
   for series in seriesList:
     newValues = []
     for val in series:
       if val is None:
         newValues.append(None)
+      elif val <= 0:
+        newValues.append(None)
       else:
         newValues.append(math.log(val, base))
     newName = "log(%s, %s)" % (series.name, base)
@@ -304,61 +375,97 @@ def log(seriesList, base=10):
     results.append(newSeries)
   return results
 
-def maximumAbove(seriesList, n):
+
+def maximumAbove(requestContext, seriesList, n):
   results = []
   for series in seriesList:
     if max(series) >= n:
       results.append(series)
   return results
 
-def maximumBelow(seriesList, n):
+
+def maximumBelow(requestContext, seriesList, n):
   result = []
   for series in seriesList:
     if max(series) <= n:
       result.append(series)
   return result
 
-def highestCurrent(seriesList, n):
+
+def highestCurrent(requestContext, seriesList, n):
   return sorted( seriesList, key=safeLast )[-n:]
 
-def lowestCurrent(seriesList, n):
+def highestMax(requestContext, seriesList, n):
+  """Returns upto n seriesList members where the respective series has a max member is in the top-n."""
+  result_list = sorted( seriesList, key=lambda s: max(s) )[-n:]
+
+  return sorted(result_list, key=lambda s: max(s), reverse=True)
+
+def lowestCurrent(requestContext, seriesList, n):
   return sorted( seriesList, key=safeLast )[:n]
 
-def currentAbove(seriesList, n):
+def currentAbove(requestContext, seriesList, n):
   return [ series for series in seriesList if safeLast(series) >= n ]
 
-def currentBelow(seriesList, n):
+def currentBelow(requestContext, seriesList, n):
   return [ series for series in seriesList if safeLast(series) <= n ]
 
-def highestAverage(seriesList, n):
+def highestAverage(requestContext, seriesList, n):
   return sorted( seriesList, key=lambda s: safeDiv(safeSum(s),safeLen(s)) )[-n:]
 
-def lowestAverage(seriesList, n):
+def lowestAverage(requestContext, seriesList, n):
   return sorted( seriesList, key=lambda s: safeDiv(safeSum(s),safeLen(s)) )[:n]
 
-def averageAbove(seriesList, n):
+def averageAbove(requestContext, seriesList, n):
   return [ series for series in seriesList if safeDiv(safeSum(series),safeLen(series)) >= n ]
 
-def averageBelow(seriesList, n):
+def averageBelow(requestContext, seriesList, n):
   return [ series for series in seriesList if safeDiv(safeSum(series),safeLen(series)) <= n ]
 
-def limit(seriesList, n):
+def percentileOrdinal(n, series):
+  result = int( safeDiv(n * len(series), 100) + 0.5 )
+  return result
+
+def nPercentile(requestContext, seriesList, n):
+  """Returns n-percent of each series in the seriesList."""
+  assert n, 'The requested percent is required to be greater than 0'
+
+  results = []
+  for s in seriesList:
+    # Create a sorted copy of the TimeSeries excluding None values in the values list.
+    s_copy = TimeSeries( s.name, s.start, s.end, s.step, sorted( [item for item in s if item is not None] ) )
+    if not s_copy:
+      continue  # Skip this series because it is empty.
+
+    pord = percentileOrdinal( n, s_copy )
+    if pord > 0:
+      i = pord - 1
+    else:
+      i = pord
+
+    perc_val = s_copy[i]
+    if perc_val:
+      results.append( TimeSeries( '%dth Percentile(%s, %.1f)' % ( n, s_copy.name, perc_val ),
+                                  s_copy.start, s_copy.end, s_copy.step, [perc_val] ) )
+  return results
+
+def limit(requestContext, seriesList, n):
   return seriesList[0:n]
 
-def sortByMaxima(seriesList):
+def sortByMaxima(requestContext, seriesList):
   def compare(x,y):
     return cmp(max(y), max(x))
   seriesList.sort(compare)
   return seriesList
 
-def sortByMinima(seriesList):
+def sortByMinima(requestContext, seriesList):
   def compare(x,y):
     return cmp(min(x), min(y))
   newSeries = [series for series in seriesList if max(series) > 0]
   newSeries.sort(compare)
   return newSeries
 
-def mostDeviant(n, seriesList):
+def mostDeviant(requestContext, n, seriesList):
   deviants = []
   for series in seriesList:
     mean = safeDiv( safeSum(series), safeLen(series) )
@@ -370,52 +477,66 @@ def mostDeviant(n, seriesList):
   deviants.sort(key=lambda i: i[0], reverse=True) #sort by sigma
   return [ series for (sigma,series) in deviants ][:n] #return the n most deviant series
 
+
 # returns a two-element tuple
 # the first element is the std dev, the second is the new sum of squares
 def doStdDev(sumOfSquares, first, new, n, avg):
    newSumOfSquares = sumOfSquares - (first * first) + (new * new)
    return (math.sqrt((newSumOfSquares / float(n)) - (avg * avg)), newSumOfSquares)
 
-def stdev(seriesList,time):
+
+def stdev(requestContext, seriesList, time):
   count = 0
   for series in seriesList:
-    stddevs = TimeSeries("stddev(%s,%.1f)" % (series.name,float(time)),series.start,series.end,series.step,[])
-    stddevs.pathExpression = "stddev(%s,%.1f)" % (series.name,float(time))
+    stddevs = TimeSeries("stddev(%s,%.1f)" % (series.name, float(time)), series.start, series.end, series.step, [])
+    stddevs.pathExpression = "stddev(%s,%.1f)" % (series.name, float(time))
     avg = safeDiv(safeSum(series[:time]), time)
 
-    sumOfSquares = sum(map(lambda(x): x * x, series[:time]))
-    (sd, sumOfSquares) = doStdDev(sumOfSquares, 0, 0, time, avg)
-    stddevs.append(sd)
+    if avg is not None:
+      sumOfSquares = sum(map(lambda(x): x * x, [v for v in series[:time] if v is not None]))
+      (sd, sumOfSquares) = doStdDev(sumOfSquares, 0, 0, time, avg)
+      stddevs.append(sd)
+    else:
+      stddevs.append(None)
 
     for (index, el) in enumerate(series[time:]):
       if el is None:
         continue
+
       toDrop = series[index]
       if toDrop is None:
         toDrop = 0
+
       s = safeSum([safeMul(time, avg), el, -toDrop])
       avg = safeDiv(s, time)
 
-      (sd, sumOfSquares) = doStdDev(sumOfSquares, toDrop, series[index+time], time, avg)
-      stddevs.append(sd)
+      if avg is not None:
+        (sd, sumOfSquares) = doStdDev(sumOfSquares, toDrop, series[index+time], time, avg)
+        stddevs.append(sd)
+      else:
+        stddevs.append(None)
+
     for i in range(0, time-1):
       stddevs.insert(0, None)
+
     seriesList[count] = stddevs
     count = count + 1
+
   return seriesList
 
-def drawAsInfinite(seriesList):
+
+def drawAsInfinite(requestContext, seriesList):
   for series in seriesList:
     series.options['drawAsInfinite'] = True
     series.name = 'drawAsInfinite(%s)' % series.name
   return seriesList
 
-def lineWidth(seriesList, width):
+def lineWidth(requestContext, seriesList, width):
   for series in seriesList:
     series.options['lineWidth'] = width
   return seriesList
 
-def dashed(*seriesList):
+def dashed(requestContext, *seriesList):
   if len(seriesList) == 2:
     dashLength = seriesList[1]
   else:
@@ -425,18 +546,166 @@ def dashed(*seriesList):
     series.options['dashed'] = dashLength
   return seriesList[0]
 
-def timeShift(seriesList, timeShift):
-  for series in seriesList:
-    series.name = 'timeShift(%s, %s)' % (series.name, timeShift)
-  return seriesList
 
-def pieAverage(series):
+def timeShift(requestContext, seriesList, timeShift):
+  delta = abs( parseTimeOffset(timeShift) )
+  myContext = requestContext.copy()
+  myContext['startTime'] = requestContext['startTime'] - delta
+  myContext['endTime'] = requestContext['endTime'] - delta
+  series = seriesList[0] # if len(seriesList) > 1, they will all have the same pathExpression, which is all we care about.
+  results = []
+
+  for shiftedSeries in evaluateTarget(myContext, series.pathExpression):
+    shiftedSeries.name = 'timeShift(%s, %s)' % (shiftedSeries.name, timeShift)
+    shiftedSeries.start = series.start
+    shiftedSeries.end = series.end
+    results.append(shiftedSeries)
+
+  return results
+
+
+def constantLine(requestContext, value):
+  start = timestamp( requestContext['startTime'] )
+  end = timestamp( requestContext['endTime'] )
+  step = end - start
+  series = TimeSeries(str(value), start, end, step, [value])
+  return [series]
+
+
+def threshold(requestContext, value, label=None, color=None):
+  series = constantLine(requestContext, value)[0]
+  if label:
+    series.name = label
+  if color:
+    series.color = color
+
+  return [series]
+
+
+def group(requestContext, *seriesLists):
+  seriesGroup = []
+  for s in seriesLists:
+    seriesGroup.extend(s)
+
+  return seriesGroup
+
+
+def exclude(requestContext, seriesList, pattern):
+  regex = re.compile(pattern)
+  return [s for s in seriesList if not regex.search(s.name)]
+
+
+def summarize(requestContext, seriesList, intervalString):
+  results = []
+  delta = parseTimeOffset(intervalString)
+  interval = delta.seconds + (delta.days * 86400)
+
+  for series in seriesList:
+    buckets = {}
+
+    timestamps = range( int(series.start), int(series.end), int(series.step) )
+    datapoints = zip(timestamps, series)
+
+    for (timestamp, value) in datapoints:
+      bucketInterval = timestamp - (timestamp % interval)
+
+      if bucketInterval not in buckets:
+        buckets[bucketInterval] = []
+
+      if value is not None:
+        buckets[bucketInterval].append(value)
+
+    newStart = series.start - (series.start % interval)
+    newEnd = series.end - (series.end % interval) + interval
+    newValues = []
+    for timestamp in range(newStart, newEnd, interval):
+      bucket = buckets.get(timestamp, [])
+
+      if bucket:
+        newValues.append( sum(bucket) )
+      else:
+        newValues.append( None )
+
+    newName = "summarize(%s, \"%s\")" % (series.name, intervalString)
+    newSeries = TimeSeries(newName, newStart, newEnd, interval, newValues)
+    newSeries.pathExpression = newName
+    results.append(newSeries)
+
+  return results
+
+
+def hitcount(requestContext, seriesList, intervalString):
+  """Estimate hit counts from a list of time series.
+
+  This function assumes the values in each time series represent
+  hits per second.  It calculates hits per some larger interval
+  such as per day or per hour.  This function is like summarize(),
+  except that it compensates automatically for different time scales
+  (so that a similar graph results from using either fine-grained
+  or coarse-grained records) and handles rarely-occurring events
+  gracefully.
+  """
+  results = []
+  delta = parseTimeOffset(intervalString)
+  interval = int(delta.seconds + (delta.days * 86400))
+
+  for series in seriesList:
+    length = len(series)
+    step = int(series.step)
+    bucket_count = int(math.ceil(float(series.end - series.start) / interval))
+    buckets = [[] for _ in range(bucket_count)]
+    newStart = int(series.end - bucket_count * interval)
+
+    for i, value in enumerate(series):
+      if value is None:
+        continue
+
+      start_time = int(series.start + i * step)
+      start_bucket, start_mod = divmod(start_time - newStart, interval)
+      end_time = start_time + step
+      end_bucket, end_mod = divmod(end_time - newStart, interval)
+
+      if end_bucket >= bucket_count:
+        end_bucket = bucket_count - 1
+        end_mod = interval
+
+      if start_bucket == end_bucket:
+        # All of the hits go to a single bucket.
+        if start_bucket >= 0:
+          buckets[start_bucket].append(value * (end_mod - start_mod))
+
+      else:
+        # Spread the hits among 2 or more buckets.
+        if start_bucket >= 0:
+          buckets[start_bucket].append(value * (interval - start_mod))
+        hits_per_bucket = value * interval
+        for j in range(start_bucket + 1, end_bucket):
+          buckets[j].append(hits_per_bucket)
+        if end_mod > 0:
+          buckets[end_bucket].append(value * end_mod)
+
+    newValues = []
+    for bucket in buckets:
+      if bucket:
+        newValues.append( sum(bucket) )
+      else:
+        newValues.append(None)
+
+    newName = 'hitcount(%s, "%s")' % (series.name, intervalString)
+    newSeries = TimeSeries(newName, newStart, series.end, interval, newValues)
+    newSeries.pathExpression = newName
+    results.append(newSeries)
+
+  return results
+
+
+def pieAverage(requestContext, series):
   return safeDiv(safeSum(series),safeLen(series))
 
-def pieMaximum(series):
+def pieMaximum(requestContext, series):
   return max(series)
 
-def pieMinimum(series):
+def pieMinimum(requestContext, series):
   return min(series)
 
 PieFunctions = {
@@ -450,10 +719,13 @@ SeriesFunctions = {
   'sumSeries' : sumSeries,
   'sum' : sumSeries,
   'diffSeries' : diffSeries,
+  'divideSeries' : divideSeries,
   'averageSeries' : averageSeries,
   'avg' : averageSeries,
   'sumSeriesWithWildcards': sumSeriesWithWildcards,
   'averageSeriesWithWildcards': averageSeriesWithWildcards,
+  'minSeries' : minSeries,
+  'maxSeries' : maxSeries,
 
   # Transform functions
   'scale' : scale,
@@ -463,6 +735,8 @@ SeriesFunctions = {
   'nonNegativeDerivative' : nonNegativeDerivative,
   'log' : log,
   'timeShift': timeShift,
+  'summarize' : summarize,
+  'hitcount'  : hitcount,
 
   # Calculate functions
   'movingAverage' : movingAverage,
@@ -474,6 +748,7 @@ SeriesFunctions = {
   'mostDeviant' : mostDeviant,
   'highestCurrent' : highestCurrent,
   'lowestCurrent' : lowestCurrent,
+  'highestMax' : highestMax,
   'currentAbove' : currentAbove,
   'currentBelow' : currentBelow,
   'highestAverage' : highestAverage,
@@ -482,6 +757,7 @@ SeriesFunctions = {
   'averageBelow' : averageBelow,
   'maximumAbove' : maximumAbove,
   'maximumBelow' : maximumBelow,
+  'nPercentile' : nPercentile,
   'limit' : limit,
   'sortByMaxima' : sortByMaxima,
   'sortByMinima' : sortByMinima,
@@ -494,4 +770,12 @@ SeriesFunctions = {
   'lineWidth' : lineWidth,
   'dashed' : dashed,
   'substr' : substr,
+  'group' : group,
+  'exclude' : exclude,
+  'constantLine' : constantLine,
+  'threshold' : threshold,
 }
+
+
+#Avoid import circularity
+from graphite.render.evaluator import evaluateTarget
