@@ -19,13 +19,14 @@ from django.contrib import admin
 admin.autodiscover()
 
 urlpatterns = patterns('',
-  ('^admin/(.*)', admin.site.root),
+  ('^admin/', include(admin.site.urls)),
   ('^render/?', include('graphite.render.urls')),
   ('^cli/?', include('graphite.cli.urls')),
   ('^composer/?', include('graphite.composer.urls')),
   ('^metrics/?', include('graphite.metrics.urls')),
   ('^browser/?', include('graphite.browser.urls')),
   ('^account/?', include('graphite.account.urls')),
+  ('^dashboard/?', include('graphite.dashboard.urls')),
   ('^whitelist/?', include('graphite.whitelist.urls')),
   ('^content/(?P<path>.*)$', 'django.views.static.serve', {'document_root' : settings.CONTENT_DIR}),
   ('', include('graphite.browser.urls')),
